@@ -8,19 +8,23 @@ Trimesh, Scene, PointCloud, and Path objects.
 Works on all major platforms: Windows, Linux, and OSX.
 """
 import collections
-import numpy as np
 
+import numpy as np
 import pyglet
 
+# pyglet 2.0 is close to a re-write moving from fixed-function
+# to shaders and we will likely support it by forking an entirely
+# new viewer `trimesh.viewer.shaders` and then basically keeping
+# `windowed` around for backwards-compatibility with no changes
+if int(pyglet.version.split(".")[0]) >= 2:
+    raise ImportError('`trimesh.viewer.windowed` requires `pip install "pyglet<2"`')
+
+from .. import rendering, util
+from ..transformations import translation_matrix
+from ..visual import to_rgba
 from .trackball import Trackball
 
-from .. import util
-from .. import rendering
-
-from ..visual import to_rgba
-from ..transformations import translation_matrix
-
-pyglet.options['shadow_window'] = False
+pyglet.options["shadow_window"] = False
 
 import pyglet.gl as gl  # NOQA
 
@@ -29,25 +33,26 @@ _SMOOTH_MAX_FACES = 100000
 
 
 class SceneViewer(pyglet.window.Window):
-
-    def __init__(self,
-                 scene,
-                 smooth=True,
-                 flags=None,
-                 visible=True,
-                 resolution=None,
-                 start_loop=True,
-                 callback=None,
-                 callback_period=None,
-                 caption=None,
-                 fixed=None,
-                 offset_lines=True,
-                 line_settings=None,
-                 background=None,
-                 window_conf=None,
-                 profile=False,
-                 record=False,
-                 **kwargs):
+    def __init__(
+        self,
+        scene,
+        smooth=True,
+        flags=None,
+        visible=True,
+        resolution=None,
+        start_loop=True,
+        callback=None,
+        callback_period=None,
+        caption=None,
+        fixed=None,
+        offset_lines=True,
+        line_settings=None,
+        background=None,
+        window_conf=None,
+        profile=False,
+        record=False,
+        **kwargs,
+    ):
         """
         Create a window that will display a trimesh.Scene object
         in an OpenGL context via pyglet.
@@ -104,7 +109,8 @@ class SceneViewer(pyglet.window.Window):
 
         # a transform to offset lines slightly to avoid Z-fighting
         self._line_offset = translation_matrix(
-            [0, 0, scene.scale / 1000 if self.offset_lines else 0])
+            [0, 0, scene.scale / 1000 if self.offset_lines else 0]
+        )
 
         self.reset_view()
         self.batch = pyglet.graphics.Batch()
@@ -113,12 +119,13 @@ class SceneViewer(pyglet.window.Window):
         self._profile = bool(profile)
         if self._profile:
             from pyinstrument import Profiler
+
             self.Profiler = Profiler
 
         self._record = bool(record)
         if self._record:
             # will save bytes here
-            self.scene.metadata['recording'] = []
+            self.scene.metadata["recording"] = []
 
         # store kwargs
         self.kwargs = kwargs
@@ -149,8 +156,7 @@ class SceneViewer(pyglet.window.Window):
         # set the default line settings to a fraction
         # of our resolution so the points aren't tiny
         scale = max(resolution)
-        self.line_settings = {'point_size': scale / 200,
-                              'line_width': scale / 400}
+        self.line_settings = {"point_size": scale / 200, "line_width": scale / 400}
         # if we've been passed line settings override the default
         if line_settings is not None:
             self.line_settings.update(line_settings)
@@ -160,32 +166,37 @@ class SceneViewer(pyglet.window.Window):
             try:
                 # try enabling antialiasing
                 # if you have a graphics card this will probably work
-                conf = gl.Config(sample_buffers=1,
-                                 samples=4,
-                                 depth_size=24,
-                                 double_buffer=True)
-                super(SceneViewer, self).__init__(config=conf,
-                                                  visible=visible,
-                                                  resizable=True,
-                                                  width=resolution[0],
-                                                  height=resolution[1],
-                                                  caption=caption)
+                conf = gl.Config(
+                    sample_buffers=1, samples=4, depth_size=24, double_buffer=True
+                )
+                super().__init__(
+                    config=conf,
+                    visible=visible,
+                    resizable=True,
+                    width=resolution[0],
+                    height=resolution[1],
+                    caption=caption,
+                )
             except pyglet.window.NoSuchConfigException:
                 conf = gl.Config(double_buffer=True)
-                super(SceneViewer, self).__init__(config=conf,
-                                                  resizable=True,
-                                                  visible=visible,
-                                                  width=resolution[0],
-                                                  height=resolution[1],
-                                                  caption=caption)
+                super().__init__(
+                    config=conf,
+                    resizable=True,
+                    visible=visible,
+                    width=resolution[0],
+                    height=resolution[1],
+                    caption=caption,
+                )
         else:
             # window config was manually passed
-            super(SceneViewer, self).__init__(config=window_conf,
-                                              resizable=True,
-                                              visible=visible,
-                                              width=resolution[0],
-                                              height=resolution[1],
-                                              caption=caption)
+            super().__init__(
+                config=window_conf,
+                resizable=True,
+                visible=visible,
+                width=resolution[0],
+                height=resolution[1],
+                caption=caption,
+            )
 
         # add scene geometry to viewer geometry
         self._update_vertex_list()
@@ -206,8 +217,7 @@ class SceneViewer(pyglet.window.Window):
             # set up a do-nothing periodic task which will
             # trigger `self.on_draw` every `callback_period`
             # seconds if someone has passed a callback
-            pyglet.clock.schedule_interval(lambda x: x,
-                                           callback_period)
+            pyglet.clock.schedule_interval(lambda x: x, callback_period)
         if start_loop:
             pyglet.app.run()
 
@@ -221,9 +231,7 @@ class SceneViewer(pyglet.window.Window):
                 continue
             if geometry_hash(geom) == self.vertex_list_hash.get(name):
                 continue
-            self.add_geometry(name=name,
-                              geometry=geom,
-                              smooth=bool(self._smooth))
+            self.add_geometry(name=name, geometry=geom, smooth=bool(self._smooth))
 
     def _update_meshes(self):
         # call the callback if specified
@@ -249,8 +257,7 @@ class SceneViewer(pyglet.window.Window):
             # convert geometry to constructor args
             args = rendering.convert_to_vertexlist(geometry, **kwargs)
         except BaseException:
-            util.log.warning('failed to add geometry `{}`'.format(name),
-                             exc_info=True)
+            util.log.warning(f"failed to add geometry `{name}`", exc_info=True)
             return
 
         # create the indexed vertex list
@@ -261,14 +268,14 @@ class SceneViewer(pyglet.window.Window):
         self.vertex_list_mode[name] = args[1]
 
         # get the visual if the element has it
-        visual = getattr(geometry, 'visual', None)
-        if hasattr(visual, 'uv') and hasattr(visual, 'material'):
+        visual = getattr(geometry, "visual", None)
+        if hasattr(visual, "uv") and hasattr(visual, "material"):
             try:
                 tex = rendering.material_to_texture(visual.material)
                 if tex is not None:
                     self.textures[name] = tex
             except BaseException:
-                util.log.warning('failed to load texture', exc_info=True)
+                util.log.warning("failed to load texture", exc_info=True)
 
     def cleanup_geometries(self):
         """
@@ -278,11 +285,9 @@ class SceneViewer(pyglet.window.Window):
         # shorthand to scene graph
         graph = self.scene.graph
         # which parts of the graph still have geometry
-        geom_keep = set([graph[node][1] for
-                         node in graph.nodes_geometry])
+        geom_keep = {graph[node][1] for node in graph.nodes_geometry}
         # which geometries no longer need to be kept
-        geom_delete = [geom for geom in self.vertex_list
-                       if geom not in geom_keep]
+        geom_delete = [geom for geom in self.vertex_list if geom not in geom_keep]
         for geom in geom_delete:
             # remove stored vertex references
             self.vertex_list.pop(geom, None)
@@ -325,16 +330,18 @@ class SceneViewer(pyglet.window.Window):
           e.g. {'cull': False}
         """
         self.view = {
-            'cull': True,
-            'axis': False,
-            'grid': False,
-            'fullscreen': False,
-            'wireframe': False,
-            'ball': Trackball(
+            "cull": True,
+            "axis": False,
+            "grid": False,
+            "fullscreen": False,
+            "wireframe": False,
+            "ball": Trackball(
                 pose=self._initial_camera_transform,
                 size=self.scene.camera.resolution,
                 scale=self.scene.scale,
-                target=self.scene.centroid)}
+                target=self.scene.centroid,
+            ),
+        }
         try:
             # if any flags are passed override defaults
             if isinstance(flags, dict):
@@ -383,9 +390,6 @@ class SceneViewer(pyglet.window.Window):
         Enable depth test in OpenGL using distances
         from `scene.camera`.
         """
-        # set the culling depth from our camera object
-        gl.glDepthRange(camera.z_near, camera.z_far)
-
         gl.glClearDepth(1.0)
         gl.glEnable(gl.GL_DEPTH_TEST)
         gl.glDepthFunc(gl.GL_LEQUAL)
@@ -396,34 +400,33 @@ class SceneViewer(pyglet.window.Window):
     @staticmethod
     def _gl_enable_color_material():
         # do some openGL things
-        gl.glColorMaterial(gl.GL_FRONT_AND_BACK,
-                           gl.GL_AMBIENT_AND_DIFFUSE)
+        gl.glColorMaterial(gl.GL_FRONT_AND_BACK, gl.GL_AMBIENT_AND_DIFFUSE)
         gl.glEnable(gl.GL_COLOR_MATERIAL)
         gl.glShadeModel(gl.GL_SMOOTH)
 
-        gl.glMaterialfv(gl.GL_FRONT,
-                        gl.GL_AMBIENT,
-                        rendering.vector_to_gl(
-                            0.192250, 0.192250, 0.192250))
-        gl.glMaterialfv(gl.GL_FRONT,
-                        gl.GL_DIFFUSE,
-                        rendering.vector_to_gl(
-                            0.507540, 0.507540, 0.507540))
-        gl.glMaterialfv(gl.GL_FRONT,
-                        gl.GL_SPECULAR,
-                        rendering.vector_to_gl(
-                            .5082730, .5082730, .5082730))
+        gl.glMaterialfv(
+            gl.GL_FRONT,
+            gl.GL_AMBIENT,
+            rendering.vector_to_gl(0.192250, 0.192250, 0.192250),
+        )
+        gl.glMaterialfv(
+            gl.GL_FRONT,
+            gl.GL_DIFFUSE,
+            rendering.vector_to_gl(0.507540, 0.507540, 0.507540),
+        )
+        gl.glMaterialfv(
+            gl.GL_FRONT,
+            gl.GL_SPECULAR,
+            rendering.vector_to_gl(0.5082730, 0.5082730, 0.5082730),
+        )
 
-        gl.glMaterialf(gl.GL_FRONT,
-                       gl.GL_SHININESS,
-                       .4 * 128.0)
+        gl.glMaterialf(gl.GL_FRONT, gl.GL_SHININESS, 0.4 * 128.0)
 
     @staticmethod
     def _gl_enable_blending():
         # enable blending for transparency
         gl.glEnable(gl.GL_BLEND)
-        gl.glBlendFunc(gl.GL_SRC_ALPHA,
-                       gl.GL_ONE_MINUS_SRC_ALPHA)
+        gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)
 
     @staticmethod
     def _gl_enable_smooth_lines(line_width=4, point_size=4):
@@ -445,16 +448,15 @@ class SceneViewer(pyglet.window.Window):
         # opengl only supports 7 lights?
         for i, light in enumerate(scene.lights[:7]):
             # the index of which light we have
-            lightN = eval('gl.GL_LIGHT{}'.format(i))
+            lightN = eval(f"gl.GL_LIGHT{i}")
 
             # get the transform for the light by name
             matrix = scene.graph.get(light.name)[0]
 
             # convert light object to glLightfv calls
             multiargs = rendering.light_to_gl(
-                light=light,
-                transform=matrix,
-                lightN=lightN)
+                light=light, transform=matrix, lightN=lightN
+            )
 
             # enable the light in question
             gl.glEnable(lightN)
@@ -470,7 +472,7 @@ class SceneViewer(pyglet.window.Window):
         non- watertight meshes you probably want to be able
         to see the back sides.
         """
-        self.view['cull'] = not self.view['cull']
+        self.view["cull"] = not self.view["cull"]
         self.update_flags()
 
     def toggle_wireframe(self):
@@ -479,14 +481,14 @@ class SceneViewer(pyglet.window.Window):
 
         Good for  looking inside meshes, off by default.
         """
-        self.view['wireframe'] = not self.view['wireframe']
+        self.view["wireframe"] = not self.view["wireframe"]
         self.update_flags()
 
     def toggle_fullscreen(self):
         """
         Toggle between fullscreen and windowed mode.
         """
-        self.view['fullscreen'] = not self.view['fullscreen']
+        self.view["fullscreen"] = not self.view["fullscreen"]
         self.update_flags()
 
     def toggle_axis(self):
@@ -495,11 +497,11 @@ class SceneViewer(pyglet.window.Window):
         off, world frame, every frame
         """
         # cycle through three axis states
-        states = [False, 'world', 'all', 'without_world']
+        states = [False, "world", "all", "without_world"]
         # the state after toggling
-        index = (states.index(self.view['axis']) + 1) % len(states)
+        index = (states.index(self.view["axis"]) + 1) % len(states)
         # update state to next index
-        self.view['axis'] = states[index]
+        self.view["axis"] = states[index]
         # perform gl actions
         self.update_flags()
 
@@ -508,7 +510,7 @@ class SceneViewer(pyglet.window.Window):
         Toggle a rendered grid.
         """
         # update state to next index
-        self.view['grid'] = not self.view['grid']
+        self.view["grid"] = not self.view["grid"]
         # perform gl actions
         self.update_flags()
 
@@ -517,24 +519,25 @@ class SceneViewer(pyglet.window.Window):
         Check the view flags, and call required GL functions.
         """
         # view mode, filled vs wirefrom
-        if self.view['wireframe']:
+        if self.view["wireframe"]:
             gl.glPolygonMode(gl.GL_FRONT_AND_BACK, gl.GL_LINE)
         else:
             gl.glPolygonMode(gl.GL_FRONT_AND_BACK, gl.GL_FILL)
 
         # set fullscreen or windowed
-        self.set_fullscreen(fullscreen=self.view['fullscreen'])
+        self.set_fullscreen(fullscreen=self.view["fullscreen"])
 
         # backface culling on or off
-        if self.view['cull']:
+        if self.view["cull"]:
             gl.glEnable(gl.GL_CULL_FACE)
         else:
             gl.glDisable(gl.GL_CULL_FACE)
 
         # case where we WANT an axis and NO vertexlist
         # is stored internally
-        if self.view['axis'] and self._axis is None:
+        if self.view["axis"] and self._axis is None:
             from .. import creation
+
             # create an axis marker sized relative to the scene
             axis = creation.axis(origin_size=self.scene.scale / 100)
             # create ordered args for a vertex list
@@ -543,16 +546,17 @@ class SceneViewer(pyglet.window.Window):
             self._axis = self.batch.add_indexed(*args)
         # case where we DON'T want an axis but a vertexlist
         # IS stored internally
-        elif not self.view['axis'] and self._axis is not None:
+        elif not self.view["axis"] and self._axis is not None:
             # remove the axis from the rendering batch
             self._axis.delete()
             # set the reference to None
             self._axis = None
 
-        if self.view['grid'] and self._grid is None:
+        if self.view["grid"] and self._grid is None:
             try:
                 # create a grid marker
                 from ..path.creation import grid
+
                 bounds = self.scene.bounds
                 center = bounds.mean(axis=0)
                 # set the grid to the lowest Z position
@@ -561,18 +565,14 @@ class SceneViewer(pyglet.window.Window):
                 # choose the side length by maximum XY length
                 side = bounds.ptp(axis=0)[:2].max()
                 # create an axis marker sized relative to the scene
-                grid_mesh = grid(
-                    side=side,
-                    count=4,
-                    transform=translation_matrix(center))
+                grid_mesh = grid(side=side, count=4, transform=translation_matrix(center))
                 # convert the path to vertexlist args
                 args = rendering.convert_to_vertexlist(grid_mesh)
                 # create ordered args for a vertex list
                 self._grid = self.batch.add_indexed(*args)
             except BaseException:
-                util.log.warning(
-                    'failed to create grid!', exc_info=True)
-        elif not self.view['grid'] and self._grid is not None:
+                util.log.warning("failed to create grid!", exc_info=True)
+        elif not self.view["grid"] and self._grid is not None:
             self._grid.delete()
             self._grid = None
 
@@ -594,10 +594,9 @@ class SceneViewer(pyglet.window.Window):
         camera = self.scene.camera
 
         # set perspective from camera data
-        gl.gluPerspective(camera.fov[1],
-                          width / float(height),
-                          camera.z_near,
-                          camera.z_far)
+        gl.gluPerspective(
+            camera.fov[1], width / float(height), camera.z_near, camera.z_far
+        )
         gl.glMatrixMode(gl.GL_MODELVIEW)
 
         return width, height
@@ -608,44 +607,44 @@ class SceneViewer(pyglet.window.Window):
         """
         width, height = self._update_perspective(width, height)
         self.scene.camera.resolution = (width, height)
-        self.view['ball'].resize(self.scene.camera.resolution)
-        self.scene.camera_transform[...] = self.view['ball'].pose
+        self.view["ball"].resize(self.scene.camera.resolution)
+        self.scene.camera_transform = self.view["ball"].pose
 
     def on_mouse_press(self, x, y, buttons, modifiers):
         """
         Set the start point of the drag.
         """
-        self.view['ball'].set_state(Trackball.STATE_ROTATE)
-        if (buttons == pyglet.window.mouse.LEFT):
-            ctrl = (modifiers & pyglet.window.key.MOD_CTRL)
-            shift = (modifiers & pyglet.window.key.MOD_SHIFT)
-            if (ctrl and shift):
-                self.view['ball'].set_state(Trackball.STATE_ZOOM)
+        self.view["ball"].set_state(Trackball.STATE_ROTATE)
+        if buttons == pyglet.window.mouse.LEFT:
+            ctrl = modifiers & pyglet.window.key.MOD_CTRL
+            shift = modifiers & pyglet.window.key.MOD_SHIFT
+            if ctrl and shift:
+                self.view["ball"].set_state(Trackball.STATE_ZOOM)
             elif shift:
-                self.view['ball'].set_state(Trackball.STATE_ROLL)
+                self.view["ball"].set_state(Trackball.STATE_ROLL)
             elif ctrl:
-                self.view['ball'].set_state(Trackball.STATE_PAN)
-        elif (buttons == pyglet.window.mouse.MIDDLE):
-            self.view['ball'].set_state(Trackball.STATE_PAN)
-        elif (buttons == pyglet.window.mouse.RIGHT):
-            self.view['ball'].set_state(Trackball.STATE_ZOOM)
+                self.view["ball"].set_state(Trackball.STATE_PAN)
+        elif buttons == pyglet.window.mouse.MIDDLE:
+            self.view["ball"].set_state(Trackball.STATE_PAN)
+        elif buttons == pyglet.window.mouse.RIGHT:
+            self.view["ball"].set_state(Trackball.STATE_ZOOM)
 
-        self.view['ball'].down(np.array([x, y]))
-        self.scene.camera_transform[...] = self.view['ball'].pose
+        self.view["ball"].down(np.array([x, y]))
+        self.scene.camera_transform = self.view["ball"].pose
 
     def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
         """
         Pan or rotate the view.
         """
-        self.view['ball'].drag(np.array([x, y]))
-        self.scene.camera_transform[...] = self.view['ball'].pose
+        self.view["ball"].drag(np.array([x, y]))
+        self.scene.camera_transform = self.view["ball"].pose
 
     def on_mouse_scroll(self, x, y, dx, dy):
         """
         Zoom the view.
         """
-        self.view['ball'].scroll(dy)
-        self.scene.camera_transform[...] = self.view['ball'].pose
+        self.view["ball"].scroll(dy)
+        self.scene.camera_transform = self.view["ball"].pose
 
     def on_key_press(self, symbol, modifiers):
         """
@@ -670,20 +669,21 @@ class SceneViewer(pyglet.window.Window):
             self.toggle_fullscreen()
 
         if symbol in [
-                pyglet.window.key.LEFT,
-                pyglet.window.key.RIGHT,
-                pyglet.window.key.DOWN,
-                pyglet.window.key.UP]:
-            self.view['ball'].down([0, 0])
+            pyglet.window.key.LEFT,
+            pyglet.window.key.RIGHT,
+            pyglet.window.key.DOWN,
+            pyglet.window.key.UP,
+        ]:
+            self.view["ball"].down([0, 0])
             if symbol == pyglet.window.key.LEFT:
-                self.view['ball'].drag([-magnitude, 0])
+                self.view["ball"].drag([-magnitude, 0])
             elif symbol == pyglet.window.key.RIGHT:
-                self.view['ball'].drag([magnitude, 0])
+                self.view["ball"].drag([magnitude, 0])
             elif symbol == pyglet.window.key.DOWN:
-                self.view['ball'].drag([0, -magnitude])
+                self.view["ball"].drag([0, -magnitude])
             elif symbol == pyglet.window.key.UP:
-                self.view['ball'].drag([0, magnitude])
-            self.scene.camera_transform[...] = self.view['ball'].pose
+                self.view["ball"].drag([0, magnitude])
+            self.scene.camera_transform = self.view["ball"].pose
 
     def on_draw(self):
         """
@@ -712,7 +712,7 @@ class SceneViewer(pyglet.window.Window):
         count = -1
 
         # if we are rendering an axis marker at the world
-        if self._axis and not self.view['axis'] == 'without_world':
+        if self._axis and not self.view["axis"] == "without_world":
             # we stored it as a vertex list
             self._axis.draw(mode=gl.GL_TRIANGLES)
         if self._grid:
@@ -740,7 +740,8 @@ class SceneViewer(pyglet.window.Window):
             if self.fixed is not None and geometry_name in self.fixed:
                 # remove altered camera transform from fixed geometry
                 transform_fix = np.linalg.inv(
-                    np.dot(self._initial_camera_transform, transform_camera))
+                    np.dot(self._initial_camera_transform, transform_camera)
+                )
                 # apply the transform so the fixed geometry doesn't move
                 transform = np.dot(transform, transform_fix)
 
@@ -756,11 +757,14 @@ class SceneViewer(pyglet.window.Window):
             # the best way to do this is probably a shader but this works fine
             if mode == gl.GL_LINES:
                 # apply the offset in camera space
-                transform = util.multi_dot([
-                    transform,
-                    np.linalg.inv(transform_camera),
-                    self._line_offset,
-                    transform_camera])
+                transform = util.multi_dot(
+                    [
+                        transform,
+                        np.linalg.inv(transform_camera),
+                        self._line_offset,
+                        transform_camera,
+                    ]
+                )
 
             # add a new matrix to the model stack
             gl.glPushMatrix()
@@ -768,16 +772,18 @@ class SceneViewer(pyglet.window.Window):
             gl.glMultMatrixf(rendering.matrix_to_gl(transform))
 
             # draw an axis marker for each mesh frame
-            if self.view['axis'] == 'all':
+            if self.view["axis"] == "all":
                 self._axis.draw(mode=gl.GL_TRIANGLES)
-            elif self.view['axis'] == 'without_world':
+            elif self.view["axis"] == "without_world":
                 if not util.allclose(transform, np.eye(4), atol=1e-5):
                     self._axis.draw(mode=gl.GL_TRIANGLES)
 
             # transparent things must be drawn last
-            if (hasattr(mesh, 'visual') and
-                hasattr(mesh.visual, 'transparency')
-                    and mesh.visual.transparency):
+            if (
+                hasattr(mesh, "visual")
+                and hasattr(mesh.visual, "transparency")
+                and mesh.visual.transparency
+            ):
                 # put the current item onto the back of the queue
                 if count < count_original:
                     # add the node to be drawn last
@@ -805,17 +811,17 @@ class SceneViewer(pyglet.window.Window):
 
         if self._profile:
             profiler.stop()
-            print(profiler.output_text(unicode=True, color=True))
+            util.log.debug(profiler.output_text(unicode=True, color=True))
 
     def flip(self):
-        super(SceneViewer, self).flip()
+        super().flip()
         if self._record:
             # will save a PNG-encoded bytes
             img = self.save_image(util.BytesIO())
             # seek start of file-like object
             img.seek(0)
             # save the bytes from the file object
-            self.scene.metadata['recording'].append(img.read())
+            self.scene.metadata["recording"].append(img.read())
 
     def save_image(self, file_obj):
         """
@@ -829,7 +835,7 @@ class SceneViewer(pyglet.window.Window):
         manager = pyglet.image.get_buffer_manager()
         colorbuffer = manager.get_color_buffer()
         # if passed a string save by name
-        if hasattr(file_obj, 'write'):
+        if hasattr(file_obj, "write"):
             colorbuffer.save(file=file_obj)
         else:
             colorbuffer.save(filename=file_obj)
@@ -849,17 +855,14 @@ def geometry_hash(geometry):
     hash : str
     """
     h = str(hash(geometry))
-    if hasattr(geometry, 'visual'):
+    if hasattr(geometry, "visual"):
         # if visual properties are defined
         h += str(hash(geometry.visual))
 
     return h
 
 
-def render_scene(scene,
-                 resolution=None,
-                 visible=True,
-                 **kwargs):
+def render_scene(scene, resolution=None, visible=True, **kwargs):
     """
     Render a preview of a scene to a PNG. Note that
     whether this works or not highly variable based on
@@ -885,8 +888,8 @@ def render_scene(scene,
       Image in PNG format
     """
     window = SceneViewer(
-        scene, start_loop=False, visible=visible,
-        resolution=resolution, **kwargs)
+        scene, start_loop=False, visible=visible, resolution=resolution, **kwargs
+    )
 
     from ..util import BytesIO
 
@@ -895,7 +898,7 @@ def render_scene(scene,
         pyglet.clock.tick()
         window.switch_to()
         window.dispatch_events()
-        window.dispatch_event('on_draw')
+        window.dispatch_event("on_draw")
         window.flip()
         if save:
             # save the color buffer data to memory
